import { Component } from '@angular/core';
import { NavController, NavParams, ActionSheetController, ToastController, LoadingController } from 'ionic-angular';
import { Shake, BarcodeScanner } from 'ionic-native';
// import { Product } from '../../models/product'
import { DataProvider } from '../../providers/data-provider'
import { CameraProvider } from '../../providers/camera-provider';
import { PreferencesProvider } from '../../providers/preferences-provider';

@Component({
    selector: 'page-product-details',
    providers: [CameraProvider],
    templateUrl: 'product-details.html'
})
export class ProductDetailsPage {

    product = {
        company_id: 0,
        default_code: null,
        description: null,
        ean13: null,
        image_medium: null,
        image_small: null,
        name: null,
        list_price: 1,
        purchase_ok: true,
        qty_available: 0,
        rental: false,
        sale_ok: true,
        standard_price: 0,
        type: "product",
        website_published: false,
        remote_id: 0,
        doc_state: "created",
        attribute_line_ids: [],
        write_date: null,
        rev: null
    }

    attribute_lines: Array<any> = [];
    attribute_values: Array<any> = [];
    attributes: Array<any> = [];
    view: string = "information";
    watcher: any;

    constructor(
        public navCtrl: NavController,
        public params: NavParams,
        public data: DataProvider,
        public toastCtrl: ToastController,
        public actionSheetCtrl: ActionSheetController,
        public loadingCtrl: LoadingController,
        public cameraProvider: CameraProvider,
        public preferencesPreferences: PreferencesProvider
    ) { 
        if (!(this.params.data instanceof Array)) {
            this.product = this.params.data;
            this.product.doc_state = "updated";

            delete this.product.rev;
        }
    }

    /**
     *
     */
    ionViewDidLoad() {
        this.initialize();

        this.watcher = Shake.startWatch(40).subscribe(() => { 
            BarcodeScanner.scan().then(bar => { 
                this.product.ean13 = bar.cancelled ? null : bar.text;
            }).catch(e => { 
                this.toastCtrl.create({
                    message: 'No se ha podido leer el código',
                    duration: 3000
                }).present();
            });
        });
    }

    /**
     *
     */
    ionViewDidLeave() {
        if (this.watcher) {
            this.watcher.unsubscribe();
        }
    }

    /**
     *
     */
    private initialize() {
        
    }

    /**
     *
     */
    private isValidForm() {
        if (!this.product.name) {
            return false;
        }

        return true;
    }

    /**
     *
     */
    showPhotoOptions() {
        this.actionSheetCtrl.create({
            title: "Tomar una foto",
            buttons: [
                {
                    text: "Desde la cámara",
                    icon: "camera",
                    handler: () => {
                        this.takePicture("camera");
                    }
                },
                {
                    text: "Desde la galería",
                    icon: "images",
                    handler: () => {
                        this.takePicture("album");
                    }
                },
                {
                    text: "Cancelar",
                    role: "cancel"
                }
            ]
        }).present();
    }

    /**
     *
     */
    takePicture(source: string): void {
        this.cameraProvider.getPicture(source).then(i => {
            this.product.image_medium = i;
            this.product.image_small = i;
        }).catch(e => {
            console.log(e);                    
        });
    }

    /**
     *
     */
    loadVariants(): void {
        let loader = this.loadingCtrl.create({
            content: "Cargando variantes, espere..."
        });
        loader.present();

        this.data.getAll('product_attribute_line').then(product_attribute_lines => {
            this.attribute_lines = product_attribute_lines;

            return this.data.getAll('product_attribute');
        }).then(product_attributes => {
            this.attributes = product_attributes;

            return this.data.getAll('product_attribute_value');
        }).then(product_attribute_values => {
            this.attribute_values = product_attribute_values;

            loader.dismiss();
        }).catch(e => {
            console.log(e);  
        });
    }

    /**
     *
     */
    addAttribute(): void {
        if (!this.product.attribute_line_ids) {
            this.product.attribute_line_ids = [];
        }
        
        this.product.attribute_line_ids.push({
            attribute: 1,
            value: 1
        });
    }

    /**
     *
     */
    saveAttributeValues(attribute_id, attribute_values) {
        let attribute_line = this.attribute_lines.filter(item => {
            return this.product.attribute_line_ids.indexOf(item.remote_id) > -1 && item.attribute_id === attribute_id;
        })[0];

        attribute_line.value_ids = attribute_values;

        this.data.save('product_attribute_line', attribute_line).then(pal => { 
            console.log(pal);
            
        }, e => { 
            console.log(e);
            
        });
        
    }

    /**
     *
     */
    save(): void {
        if (!this.isValidForm()) {
            return;
        }

        console.log(this.product);
        

        this.data.save('product', this.product).then(result => {
            if (this.params.data instanceof Array) {
                this.params.data.push(result.products[0]);
            }
            
            this.navCtrl.pop(this);
        }).catch(e => {
            console.log(e);
        });
  
    }
}
